# This file is part of Radicale Server - Calendar Server
# Copyright © 2014 Jean-Marc Martins
# Copyright © 2012-2017 Guillaume Ayoub
#
# This library is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Radicale.  If not, see <http://www.gnu.org/licenses/>.
"""
SQLAlchemy rights backend.

"""
import radicale.syno.utils as utils
from radicale.syno.permission import is_readable, is_writable
from radicale.syno.model import Model
from radicale.rights import BaseRights


class Rights(BaseRights):

    def __init__(self, configuration, logger):
        super().__init__(configuration, logger)
        db_url = configuration.get("syno", "database_url")
        self._model = Model(db_url, logger)

    def cleanup(self):
        self._model.cleanup()

    def authorized(self, user, path, permission):
        """Check if the user is allowed to read or write the collection.

        If ``user`` is empty, return false.

        ``path`` is sanitized.

        ``permission`` is "r" or "w".

        """

        if not user:
            return False

        sane_path = utils.get_sane_path(path)

        self.logger.debug(
            "[Authorizing] - user: {0}, path: {1}, permission: {2}".format(
                user,
                sane_path,
                permission
            )
        )

        if sane_path.lower() == user.lower():
            return True

        priv = self._model.get_privilege(user, path)

        if permission == "r" and is_readable(priv):
            return True
        elif permission == "w" and is_writable(priv):
            return True
        else:
            return False
