import radicale.syno.vobject_wrapper.utils as utils
import radicale.syno.constant as constant


class Person():
    _ATTR_FULL_NAME = '_full_name'
    _ATTR_NAME = '_name'
    _ATTR_GIVEN_NAME = '_given_name'
    _ATTR_MIDDLE_NAME = '_middle_name'
    _ATTR_FAMILY_NAME = '_family_name'
    _ATTR_ORGS = '_orgs'
    _ATTR_COMPANY = '_company'
    _ATTR_DEPARTMENT = '_department'
    _ATTR_TITLE = '_title'
    _ATTR_PHONETIC_GIVEN_NAME = '_phonetic_given_name'
    _ATTR_PHONETIC_MIDDLE_NAME = '_phonetic_middle_name'
    _ATTR_PHONETIC_FAMILY_NAME = '_phonetic_family_name'
    _ATTR_NICKNAME = '_nickname'
    _ATTR_NOTE = '_note'
    _ATTR_EMAIL_INFOS = '_email_infos'
    _ATTR_EMAILS = '_emails'
    _ATTR_PRIMARY_EMAIL = '_primary_email'
    _ATTR_TELEPHONE_INFOS = '_telephone_infos'
    _ATTR_TELEPHONES = '_telephones'
    _ATTR_PRIMARY_TELEPHONE = '_primary_telephone'
    _ATTR_ADDRESS_INFOS = '_address_infos'
    _ATTR_ADDRESSES = '_addresses'
    _ATTR_PRIMARY_ADDRESS = '_primary_address'
    _ATTR_URL_INFOS = '_url_infos'
    _ATTR_URLS = '_urls'
    _ATTR_PRIMARY_URL = '_primary_url'
    _ATTR_DATE_INFOS = '_date_infos'
    _ATTR_DATES = '_dates'
    _ATTR_PRIMARY_DATE = '_primary_date'
    _ATTR_BIRTHDAY = '_birthday'
    _ATTR_IM_INFOS = '_im_infos'
    _ATTR_IMS = '_ims'
    _ATTR_PRIMARY_IM = '_primary_im'
    _ATTR_PHOTO = '_photo'

    def __init__(self, vobject_item):
        self._vobject_item = vobject_item

    @property
    def full_name(self):
        if not hasattr(self, self._ATTR_FULL_NAME):
            setattr(
                self,
                self._ATTR_FULL_NAME,
                utils.get_props_first_val(
                    self._vobject_item,
                    constant.VCARD_PROPS_FULL_NAME
                )
            )
        return getattr(self, self._ATTR_FULL_NAME)

    def _get_name(self):
        if not hasattr(self, self._ATTR_NAME):
            setattr(
                self,
                self._ATTR_NAME,
                utils.get_props_first_val(
                    self._vobject_item,
                    constant.VCARD_PROPS_NAMES
                )
            )
        return getattr(self, self._ATTR_NAME)

    @property
    def given_name(self):
        if not hasattr(self, self._ATTR_GIVEN_NAME):
            name = self._get_name()
            setattr(self, self._ATTR_GIVEN_NAME, name.given if name else "")
        return getattr(self, self._ATTR_GIVEN_NAME)

    @property
    def middle_name(self):
        if not hasattr(self, self._ATTR_MIDDLE_NAME):
            name = self._get_name()
            setattr(
                self,
                self._ATTR_MIDDLE_NAME,
                name.additional if name else None
            )
        return getattr(self, self._ATTR_MIDDLE_NAME)

    @property
    def family_name(self):
        if not hasattr(self, self._ATTR_FAMILY_NAME):
            name = self._get_name()
            setattr(self, self._ATTR_FAMILY_NAME, name.family if name else None)
        return getattr(self, self._ATTR_FAMILY_NAME)

    def _get_orgs(self):
        if not hasattr(self, self._ATTR_ORGS):
            orgs = utils.get_props_first_val(
                self._vobject_item,
                constant.VCARD_PROPS_ORG
            )
            setattr(self, self._ATTR_ORGS, orgs if orgs else [])
        return getattr(self, self._ATTR_ORGS)

    @property
    def company(self):
        if not hasattr(self, self._ATTR_COMPANY):
            orgs = self._get_orgs()
            setattr(self, self._ATTR_COMPANY, orgs[0] if orgs else "")
        return getattr(self, self._ATTR_COMPANY)

    @property
    def department(self):
        if not hasattr(self, self._ATTR_DEPARTMENT):
            orgs = self._get_orgs()
            setattr(self, self._ATTR_DEPARTMENT, ', '.join(orgs[1:6]))
        return getattr(self, self._ATTR_DEPARTMENT)

    @property
    def title(self):
        if not hasattr(self, self._ATTR_TITLE):
            title = utils.get_props_first_val(
                self._vobject_item,
                constant.VCARD_PROPS_TITLE
            )
            setattr(self, self._ATTR_TITLE, title if title else "")
        return getattr(self, self._ATTR_TITLE)

    @property
    def phonetic_given_name(self):
        if not hasattr(self, self._ATTR_PHONETIC_GIVEN_NAME):
            phonetic_given_name = utils.get_props_first_val(
                self._vobject_item,
                constant.VCARD_PROPS_PHONETIC_FIRST_NAME
            )
            setattr(
                self,
                self._ATTR_PHONETIC_GIVEN_NAME,
                phonetic_given_name if phonetic_given_name else ""
            )
        return getattr(self, self._ATTR_PHONETIC_GIVEN_NAME)

    @property
    def phonetic_middle_name(self):
        if not hasattr(self, self._ATTR_PHONETIC_MIDDLE_NAME):
            phonetic_middle_name = utils.get_props_first_val(
                self._vobject_item,
                constant.VCARD_PROPS_PHONETIC_MIDDLE_NAME
            )
            setattr(
                self,
                self._ATTR_PHONETIC_MIDDLE_NAME,
                phonetic_middle_name if phonetic_middle_name else ""
            )
        return getattr(self, self._ATTR_PHONETIC_MIDDLE_NAME)

    @property
    def phonetic_family_name(self):
        if not hasattr(self, self._ATTR_PHONETIC_FAMILY_NAME):
            phonetic_family_name = utils.get_props_first_val(
                self._vobject_item,
                constant.VCARD_PROPS_PHONETIC_LAST_NAME
            )
            setattr(
                self,
                self._ATTR_PHONETIC_FAMILY_NAME,
                phonetic_family_name if phonetic_family_name else ""
            )
        return getattr(self, self._ATTR_PHONETIC_FAMILY_NAME)

    @property
    def nickname(self):
        if not hasattr(self, self._ATTR_NICKNAME):
            nickname = utils.get_props_first_val(
                self._vobject_item,
                constant.VCARD_PROPS_NICKNAME
            )
            setattr(self, self._ATTR_NICKNAME, nickname if nickname else "")
        return getattr(self, self._ATTR_NICKNAME)

    @property
    def note(self):
        if not hasattr(self, self._ATTR_NOTE):
            note = utils.get_props_first_val(
                self._vobject_item,
                constant.VCARD_PROPS_NOTE
            )
            setattr(self, self._ATTR_NOTE, note if note else "")
        return getattr(self, self._ATTR_NOTE)

    def _get_email_infos(self):
        if not hasattr(self, self._ATTR_EMAIL_INFOS):
            setattr(
                self,
                self._ATTR_EMAIL_INFOS,
                utils.get_info_items(
                    self._vobject_item,
                    constant.VCARD_PROPS_EMAIL
                )
            )
        return getattr(self, self._ATTR_EMAIL_INFOS)

    @property
    def emails(self):
        if not hasattr(self, self._ATTR_EMAILS):
            setattr(
                self,
                self._ATTR_EMAILS,
                [email.value for email in self._get_email_infos()]
            )
        return getattr(self, self._ATTR_EMAILS)

    @property
    def primary_email(self):
        if not hasattr(self, self._ATTR_PRIMARY_EMAIL):
            setattr(
                self,
                self._ATTR_PRIMARY_EMAIL,
                utils.get_primary_info_item_val(self._get_email_infos())
            )
        return getattr(self, self._ATTR_PRIMARY_EMAIL)

    def _get_telephone_infos(self):
        if not hasattr(self, self._ATTR_TELEPHONE_INFOS):
            setattr(
                self,
                self._ATTR_TELEPHONE_INFOS,
                utils.get_info_items(
                    self._vobject_item,
                    constant.VCARD_PROPS_TEL
                )
            )
        return getattr(self, self._ATTR_TELEPHONE_INFOS)

    @property
    def telephones(self):
        if not hasattr(self, self._ATTR_TELEPHONES):
            setattr(
                self,
                self._ATTR_TELEPHONES,
                [tel.value for tel in self._get_telephone_infos()]
            )
        return getattr(self, self._ATTR_TELEPHONES)

    @property
    def primary_telephone(self):
        if not hasattr(self, self._ATTR_PRIMARY_TELEPHONE):
            setattr(
                self,
                self._ATTR_PRIMARY_TELEPHONE,
                utils.get_primary_info_item_val(self._get_telephone_infos())
            )
        return getattr(self, self._ATTR_PRIMARY_TELEPHONE)

    def _get_address_infos(self):
        if not hasattr(self, self._ATTR_ADDRESS_INFOS):
            setattr(
                self,
                self._ATTR_ADDRESS_INFOS,
                utils.get_info_items(
                    self._vobject_item,
                    constant.VCARD_PROPS_ADR
                )
            )
        return getattr(self, self._ATTR_ADDRESS_INFOS)

    @property
    def addresses(self):
        if not hasattr(self, self._ATTR_ADDRESSES):
            setattr(
                self,
                self._ATTR_ADDRESSES,
                [str(adr.value) for adr in self._get_address_infos()]
            )
        return getattr(self, self._ATTR_ADDRESSES)

    @property
    def primary_address(self):
        if not hasattr(self, self._ATTR_PRIMARY_ADDRESS):
            setattr(
                self,
                self._ATTR_PRIMARY_ADDRESS,
                utils.get_primary_info_item_val(self._get_address_infos())
            )
        return getattr(self, self._ATTR_PRIMARY_ADDRESS)

    def _get_url_infos(self):
        if not hasattr(self, self._ATTR_URL_INFOS):
            setattr(
                self,
                self._ATTR_URL_INFOS,
                utils.get_info_items(
                    self._vobject_item,
                    constant.VCARD_PROPS_URL
                )
            )
        return getattr(self, self._ATTR_URL_INFOS)

    @property
    def urls(self):
        if not hasattr(self, self._ATTR_URLS):
            setattr(
                self,
                self._ATTR_URLS,
                [url.value for url in self._get_url_infos()]
            )
        return getattr(self, self._ATTR_URLS)

    @property
    def primary_url(self):
        if not hasattr(self, self._ATTR_PRIMARY_URL):
            setattr(
                self,
                self._ATTR_PRIMARY_URL,
                utils.get_primary_info_item_val(self._get_url_infos())
            )
        return getattr(self, self._ATTR_PRIMARY_URL)

    def _get_date_infos(self):
        if not hasattr(self, self._ATTR_DATE_INFOS):
            setattr(
                self,
                self._ATTR_DATE_INFOS,
                utils.get_info_items(
                    self._vobject_item,
                    constant.VCARD_PROPS_DATE
                )
            )
        return getattr(self, self._ATTR_DATE_INFOS)

    @property
    def dates(self):
        if not hasattr(self, self._ATTR_DATES):
            setattr(
                self,
                self._ATTR_DATES,
                [date.value for date in self._get_date_infos()]
            )
        return getattr(self, self._ATTR_DATES)

    @property
    def primary_date(self):
        if not hasattr(self, self._ATTR_PRIMARY_DATE):
            setattr(
                self,
                self._ATTR_PRIMARY_DATE,
                utils.get_primary_info_item_val(self._get_date_infos())
            )
        return getattr(self, self._ATTR_PRIMARY_DATE)

    @property
    def birthday(self):
        if not hasattr(self, self._ATTR_BIRTHDAY):
            setattr(
                self,
                self._ATTR_BIRTHDAY,
                utils.get_props_first_val(
                    self._vobject_item,
                    constant.VCARD_PROPS_BDAY
                )
            )
        return getattr(self, self._ATTR_BIRTHDAY)

    def _get_im_infos(self):
        if not hasattr(self, self._ATTR_IM_INFOS):
            setattr(
                self,
                self._ATTR_IM_INFOS,
                utils.get_info_items(
                    self._vobject_item,
                    constant.VCARD_PROPS_IM
                )
            )
        return getattr(self, self._ATTR_IM_INFOS)

    @property
    def ims(self):
        if not hasattr(self, self._ATTR_IMS):
            setattr(
                self,
                self._ATTR_IMS,
                [im.value for im in self._get_im_infos()]
            )
        return getattr(self, self._ATTR_IMS)

    @property
    def primary_im(self):
        if not hasattr(self, self._ATTR_PRIMARY_IM):
            setattr(
                self,
                self._ATTR_PRIMARY_IM,
                utils.get_primary_info_item_val(self._get_im_infos())
            )
        return getattr(self, self._ATTR_PRIMARY_IM)

    @property
    def photo(self):
        """
        Photo with ENCODING='b'  stores in the form of byte-string in vobject
        we need to call behavior.encode to get the base64 encoded string,
        then transfer it to the format we wanted in metadata（'data:image/``type``;base64,``base64_encoded_string``'）
        """
        if not hasattr(self, self._ATTR_PHOTO):
            photo = next(utils.get_props_items(self._vobject_item, constant.VCARD_PROPS_PHOTO), None)
            # <Contacts> #1152 - If PHOTO field in .VCF doesn't contain TYPE
            # attribute, we will not be able to generate correct data URI.
            try:
                if photo and isinstance(photo.value, bytes):
                    photo.behavior.encode(photo)
                    photo_meta = "data:image/" + photo.type_paramlist[0].lower() + ";base64," + photo.value
                else:
                    photo_meta = None
            except Exception:
                photo_meta = None
            setattr(
                self,
                self._ATTR_PHOTO,
                photo_meta
                )
        return getattr(self, self._ATTR_PHOTO)
